package org.example.wedding_app_backend.initializer;

import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.service.WeddingGuestService;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.service.WeddingImageService;
import org.example.wedding_app_backend.wedding.service.WeddingService;
import org.example.wedding_app_backend.weddingQuiz.entity.WeddingQuizEntity;
import org.example.wedding_app_backend.weddingQuiz.service.WeddingQuizService;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.service.WhoKnowsBetterQuizService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

@Component
@RequiredArgsConstructor
public class DataInitializer implements InitializingBean {
    private final WeddingService weddingService;

    private final WeddingImageService weddingImageService;

    private final WeddingQuizService weddingQuizService;

    private final WhoKnowsBetterQuizService whoKnowsBetterQuizService;

    private final WeddingGuestService weddingGuestService;

    @Override
    public void afterPropertiesSet() throws IOException {
        WeddingEntity wedding = WeddingEntity.builder()
                .id(UUID.fromString("123e4567-e89b-12d3-a456-426614174000"))
                .name("MATIEGO i KUBUSIA")
                .eventStartDateTime(LocalDateTime.of(2025, 5, 13, 18, 0))
                .backgroundImage(readImageAsByteArray("IMG_1.jpeg"))
                .build();
        weddingService.saveWedding(wedding);

        WeddingImageEntity weddingImage1 = WeddingImageEntity.builder()
                .id(UUID.randomUUID())
                .likes(0)
                .date(LocalDateTime.now())
                .wedding(wedding)
                .image(readImageAsByteArray("IMG_1.jpeg"))
                .build();
        weddingImageService.saveWeddingImage(weddingImage1);

        WeddingImageEntity weddingImage2 = WeddingImageEntity.builder()
                .id(UUID.randomUUID())
                .likes(0)
                .date(LocalDateTime.now())
                .wedding(wedding)
                .image(readImageAsByteArray("IMG_2.jpeg"))
                .build();
        weddingImageService.saveWeddingImage(weddingImage2);

        WeddingImageEntity weddingImage3 = WeddingImageEntity.builder()
                .id(UUID.randomUUID())
                .likes(0)
                .date(LocalDateTime.now())
                .wedding(wedding)
                .image(readImageAsByteArray("IMG_3.jpeg"))
                .build();
        weddingImageService.saveWeddingImage(weddingImage3);

        WeddingQuizEntity weddingQuizEntity1 = WeddingQuizEntity.builder()
                .id(UUID.randomUUID())
                .question("Czy damy rade?")
                .answer1("zTak damy rade")
                .answer2("zNie ma szans")
                .answer3("Arka gdynia . swinia")
                .answer4("zSprzedam opla")
                .wedding(wedding)
                .build();
        weddingQuizService.save(weddingQuizEntity1);

        WeddingQuizEntity weddingQuizEntity2 = WeddingQuizEntity.builder()
                .id(UUID.randomUUID())
                .question("Arka gdynia?")
                .answer1("zTo najlepszy klub")
                .answer2("ALechia lepsza")
                .answer3("zNie")
                .answer4("zSprzedam opla")
                .wedding(wedding)
                .build();
        weddingQuizService.save(weddingQuizEntity2);

        WeddingQuizEntity weddingQuizEntity3 = WeddingQuizEntity.builder()
                .id(UUID.randomUUID())
                .question("czemu")
                .answer1("ATa")
                .answer2("zw")
                .answer3("zs")
                .answer4("zSm")
                .wedding(wedding)
                .build();
        weddingQuizService.save(weddingQuizEntity3);

        WeddingQuizEntity weddingQuizEntity4 = WeddingQuizEntity.builder()
                .id(UUID.randomUUID())
                .question("test")
                .answer1("zTa")
                .answer2("zw")
                .answer3("zs")
                .answer4("ASm")
                .wedding(wedding)
                .build();
        weddingQuizService.save(weddingQuizEntity4);

        WhoKnowsBetterQuiz whoKnowsBetterQuiz1 = WhoKnowsBetterQuiz.builder()
                .id(UUID.randomUUID())
                .question("Kto ma duże stopy?")
                .groomScore(20)
                .brideScore(15)
                .wedding(wedding)
                .build();
        whoKnowsBetterQuizService.save(whoKnowsBetterQuiz1);

        WhoKnowsBetterQuiz whoKnowsBetterQuiz2 = WhoKnowsBetterQuiz.builder()
                .id(UUID.randomUUID())
                .question("Kto ma duże nogi?")
                .groomScore(4)
                .brideScore(31)
                .wedding(wedding)
                .build();
        whoKnowsBetterQuizService.save(whoKnowsBetterQuiz2);

        WeddingGuestEntity aro1 = WeddingGuestEntity.builder()
                .id(UUID.fromString("123e4567-e89b-12d3-a456-426614174001"))
                .questionIndex(0)
                .quizPoints(0)
                .quizFinished(false)
                .quizSeed((int) (Math.random()*3)+3)
                .guestName("AREK")
                .whoKnowsQuestionIndex(1)
                .whoKnowsQuizFinished(false)
                .wedding(wedding)
                .build();
        weddingGuestService.save(aro1);

        WeddingGuestEntity aro2 = WeddingGuestEntity.builder()
                .id(UUID.fromString("123e4567-e89b-12d3-a456-426614174002"))
                .questionIndex(0)
                .quizPoints(0)
                .quizFinished(false)
                .quizSeed((int) (Math.random()*3)+3)
                .guestName("MAREK")
                .whoKnowsQuestionIndex(0)
                .whoKnowsQuizFinished(false)
                .wedding(wedding)
                .build();
        weddingGuestService.save(aro2);

        WeddingGuestEntity aro3 = WeddingGuestEntity.builder()
                .id(UUID.fromString("123e4567-e89b-12d3-a456-426614174003"))
                .questionIndex(0)
                .quizPoints(1000)
                .quizFinished(true)
                .quizSeed((int) (Math.random()*3)+3)
                .guestName("BROWAREK")
                .whoKnowsQuestionIndex(0)
                .whoKnowsQuizFinished(false)
                .wedding(wedding)
                .build();
        weddingGuestService.save(aro3);
    }


    private byte[] readImageAsByteArray(String name) throws IOException {
        try (InputStream is = this.getClass().getResourceAsStream("/images/" + name)) {
            return is.readAllBytes();
        }
    }
}