package org.example.wedding_app_backend.weddingImage.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.weddingImage.dto.GetWeddingImageResponse;
import org.example.wedding_app_backend.weddingImage.function.RequestToWeddingImageFunction;
import org.example.wedding_app_backend.weddingImage.function.WeddingImagesMapper;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.service.WeddingImageService;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.UUID;

@RequestMapping("/api/wedding")
@RestController
@Log
@RequiredArgsConstructor
public class WeddingImageController {

    private final WeddingImageService weddingImageService;

    private final RequestToWeddingImageFunction requestToWeddingImage;

    private final WeddingImagesMapper weddingImagesMapper;

    @GetMapping("{weddingId}/{guestId}/gallery")
    @PreAuthorize("hasRole('client_user')")
    public ResponseEntity<List<GetWeddingImageResponse>> getWeddingGallery(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestParam(value = "sort") String sort)
    {
        return weddingImageService.findAllWeddingImages(weddingId, guestId, sort)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping(path = "{weddingId}/gallery/{id}", produces = MediaType.IMAGE_JPEG_VALUE)
    public ResponseEntity<byte[]> getWeddingImage(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("id") UUID id)
    {
        return weddingImageService.findWeddingImage(id)
                .map(WeddingImageEntity::getImage)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @PostMapping(path = "{weddingId}/gallery", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    public void putWeddingImage(
            @PathVariable("weddingId") UUID weddingId,
            @RequestParam("image") MultipartFile request)
    {
        weddingImageService.saveWeddingImage(requestToWeddingImage.apply(weddingId, request));
    }

    @PatchMapping(path = "{weddingId}/{guestId}/gallery/{imageId}")
    public void changeWeddingImageLikes(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @PathVariable("imageId") UUID imageId,
            @RequestParam(value = "liked") boolean liked)
    {
        weddingImageService.findWeddingImage(imageId)
                .ifPresentOrElse(
                        character -> {
                            weddingImageService.updateLikes(guestId, imageId, liked);
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        }
                );
    }
}
