package org.example.wedding_app_backend.weddingQuiz.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.service.WeddingGuestService;
import org.example.wedding_app_backend.weddingQuiz.dto.GetWeddingQuizResponse;
import org.example.wedding_app_backend.weddingQuiz.function.WeddingQuizQuestionMapper;
import org.example.wedding_app_backend.weddingQuiz.service.WeddingQuizService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.Map;
import java.util.Optional;
import java.util.UUID;

@RequestMapping("/api/wedding")
@RestController
@Log
@RequiredArgsConstructor
public class WeddingQuizController {

    private final WeddingQuizService weddingQuizService;

    private final WeddingQuizQuestionMapper weddingQuizQuestionMapper;

    private final WeddingGuestService weddingGuestService;



    @GetMapping("{weddingId}/{guestId}/games/quiz")
    public ResponseEntity<GetWeddingQuizResponse> getWeddingQuestion(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingQuizService.findQuestionByGuestId(weddingId,guestId)
                .map(weddingQuizQuestionMapper::entityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @PatchMapping(path = "{weddingId}/{guestId}/games/quiz")
    public void updateWeddingGuestPoints(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestBody Map<String, Integer> payload)
    {
        Optional<WeddingGuestEntity> guest = weddingGuestService.findWeddingGuest(guestId);

        if (guest.isPresent()) {
            weddingGuestService.updateQuizPoints(guestId, payload.get("score"));
            weddingGuestService.updateQuizIndex(guestId,weddingId);
        } else {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
        }
    }


}
