package org.example.wedding_app_backend.whoKnowsBetterQuiz.service;

import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.repository.WhoKnowsBetterQuizRepository;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;
import java.util.UUID;

@Service
@RequiredArgsConstructor
public class WhoKnowsBetterQuizService {

    private final WhoKnowsBetterQuizRepository whoKnowsBetterQuizRepository;

    private final WeddingRepository weddingRepository;

    private final WeddingGuestRepository weddingGuestRepository;

    public void save(WhoKnowsBetterQuiz whoKnowsBetterQuiz) {
        whoKnowsBetterQuizRepository.save(whoKnowsBetterQuiz);
    }

    @Transactional
    public Optional<List<WhoKnowsBetterQuiz>> findAllByWedding(UUID weddingId) {

        return weddingRepository.findById(weddingId)
                .map(whoKnowsBetterQuizRepository::findAllByWedding);

    }

    @Transactional
    public Optional<WhoKnowsBetterQuiz> findQuestionByGuestId(UUID weddingId, UUID guestId) {

        WeddingGuestEntity guest = weddingGuestRepository.findById(guestId)
                .orElseThrow(() -> new NoSuchElementException("Guest with ID " + guestId + " not found."));
        return whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(weddingId, guest.getWhoKnowsQuestionIndex());

    }

    @Transactional
    public void updateQuestionCounter(UUID weddingId ,UUID guestId, String answer) {

        WeddingGuestEntity guest = weddingGuestRepository.findById(guestId)
                .orElseThrow(() -> new NoSuchElementException("Guest with ID " + guestId + " not found."));

        WhoKnowsBetterQuiz currentQuestion = whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(weddingId, guest.getWhoKnowsQuestionIndex())
                .orElseThrow(() -> new NoSuchElementException("Question with index " + guest.getWhoKnowsQuestionIndex() + " not found."));
        System.out.println("Answer: " + answer);
        if("groom".equalsIgnoreCase(answer)){
            whoKnowsBetterQuizRepository.updateGroomScore(weddingId, currentQuestion.getId());
        }else if("bride".equalsIgnoreCase(answer)){
            whoKnowsBetterQuizRepository.updateBrideScore(weddingId, currentQuestion.getId());
        }else{
            throw new IllegalArgumentException("Invalid answer");
        }

    }
}
