import {
  APP_INITIALIZER,
  ApplicationConfig, importProvidersFrom,
  provideZoneChangeDetection
} from '@angular/core';
import {provideRouter, withComponentInputBinding} from '@angular/router';

import { routes } from './app.routes';
import {HTTP_INTERCEPTORS, provideHttpClient, withFetch, withInterceptors} from '@angular/common/http';
import {KeycloakAngularModule, KeycloakBearerInterceptor, KeycloakService} from 'keycloak-angular';
import {initializeKeycloak} from './keycloak/keycloak-init';
import {keycloakInterceptor} from './interceptors/CustomKeycloakInterceptor';

export const appConfig: ApplicationConfig = {
  providers: [
    importProvidersFrom(KeycloakAngularModule),
    KeycloakService,
    provideHttpClient(
      withFetch(),
      withInterceptors([keycloakInterceptor])
    ),
    {
      provide: APP_INITIALIZER,
      useFactory: initializeKeycloak,
      multi: true,
      deps: [KeycloakService]
    },
    provideZoneChangeDetection({ eventCoalescing: true }),
    provideRouter(routes),
  ]
};
