import {Component, OnInit, signal, WritableSignal} from '@angular/core';
import {ActivatedRoute, NavigationEnd, Router, RouterLink} from '@angular/router';
import {NavigationType} from '../../models/navigation.type';
import {TabMenuModule} from 'primeng/tabmenu';

@Component({
  selector: 'app-nav',
  standalone: true,
  templateUrl: './nav.component.html',
  imports: [
    RouterLink,
    TabMenuModule
  ],
  styleUrl: './nav.component.css'
})
export class NavComponent implements OnInit {

  constructor(private router: Router,private route: ActivatedRoute) {}

  weddingId: WritableSignal<string | null> = signal<string | null>(null);

  guestId: WritableSignal<string | null> = signal<string | null>(null);

  items: Array<NavigationType> = [
    {icon: 'assets/icons/home.svg', label: 'Home',
      command: () => {
        this.router.navigate(['/wedding', this.weddingId(), this.guestId()]);
      }
    },
    {icon: 'assets/icons/picture.svg', label: 'Gallery',
      command: () => {
        this.router.navigate(['/wedding', this.weddingId() , this.guestId(), 'gallery']);
      }
    },
    {icon: 'assets/icons/games.svg', label: 'Games',
      command: () => {
        this.router.navigate(['wedding', this.weddingId() , this.guestId(), 'games']);
      }
    }]

  ngOnInit(): void {
    this.router.events.subscribe(event => {
      if (event instanceof NavigationEnd) {
        this.weddingId.set(this.route.firstChild?.snapshot.params['weddingId']);
        this.guestId.set(this.route.firstChild?.snapshot.params['guestId']);
      }
    });
  }

}
