import {Component, Input, OnInit, signal, WritableSignal} from '@angular/core';
import { QuizService } from '../../services/quiz.service';
import { QuizType } from '../../models/quiz.type';
import {CardModule} from 'primeng/card';
import {Button, ButtonDirective} from 'primeng/button';
import {NgForOf, NgIf} from '@angular/common';
import {ActivatedRoute, Router, RouterLink} from '@angular/router';
import {catchError} from 'rxjs';
import {FormsModule} from '@angular/forms';
import {HallOfFameService} from '../../services/hall-of-fame.service';
import {QuizStateType} from '../../models/quizState.type';
import {QuizScoreType} from '../../models/quizScore.type';

@Component({
  selector: 'app-quiz',
  standalone: true,
  templateUrl: './quiz.component.html',
  styleUrls: ['./quiz.component.css'],
  imports: [
    CardModule,
    ButtonDirective,
    NgIf,
    NgForOf,
    Button,
    FormsModule
  ]
})
export class QuizComponent implements OnInit {

  question: QuizType = {
    question: '',
    answer1: '',
    answer2: '',
    answer3: '',
    answer4: '',
  };
  quizCompleted: QuizStateType= {quizFinished:false};
  quizStats: QuizScoreType={
    questionIndex: 0,
    quizPoints:0,
    guestName:'',
  };

  shuffledAnswers: string[] = [];
  selectedAnswer: string = '';
  correct: boolean | null = null;

  weddingId: string = '';
  guestId:string = '';

  constructor(private router: Router,private route: ActivatedRoute,private QuizService: QuizService) {}

  ngOnInit() {
    this.weddingId = this.route.snapshot.paramMap.get('weddingId') || '';
    this.guestId = this.route.snapshot.paramMap.get('guestId') || '';
    if (this.weddingId) {
      this.nextQuestion();
    } else {
      console.error('weddingId error');
    }
  }

  onAnswerSelected(answer: string) {
    this.selectedAnswer = answer;
    this.correct = answer === this.question.answer1;
    let point=0;
    if (this.correct) {
      point++;
    }
    this.QuizService.sendPointsByGuestId(this.weddingId, this.guestId, point)
      .subscribe({
        error: (error) => console.error("Błąd podczas wysyłania punktów:", error)
      });
  }

  nextQuestion() {
    this.QuizService.getQuizStateByWeddingId(this.weddingId, this.guestId).pipe(
      catchError((error) => {
        console.error('Error fetching quiz state:', error);
        throw error;
      })
    ).subscribe((weddingQuizState) => {
      this.quizCompleted = weddingQuizState;

      if (!this.quizCompleted.quizFinished) {
        this.QuizService.getQuizByWeddingId(this.weddingId, this.guestId).pipe(
          catchError((error) => {
            console.error('Error fetching next question:', error);
            throw error;
          })
        ).subscribe((nextQuestion) => {
          this.question = nextQuestion;
          this.shuffleAnswers();
          this.selectedAnswer = '';
          this.correct = null;
        });
      } else {
          this.QuizService.getQuizScoreByWeddingId(this.weddingId, this.guestId).pipe(catchError((error) => {
            console.error('Error fetching next question:', error);
            throw error;
          })).subscribe((state) => {
            this.quizStats = state;
          });
      }
    });

  }

  onClickRoute(){
    this.router.navigateByUrl(`/wedding/${this.weddingId}/${this.guestId}/games/hall`);
  }

  shuffleAnswers(): void {
    const answers = [
      this.question.answer1,
      this.question.answer2,
      this.question.answer3,
      this.question.answer4,
    ];

    for (let i = answers.length - 1; i > 0; i--) {
      const j = Math.floor(Math.random() * (i + 1));
      [answers[i], answers[j]] = [answers[j], answers[i]];
    }

    this.shuffledAnswers = answers;
  }

}
