import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { WhoKnowsBetterQuizService } from '../../services/who-knows-better-quiz.service';
import { Question } from '../../models/whoKnowsBetter.type';
import {CommonModule} from '@angular/common';
import {WhoKnowsBetterQuizStateType} from '../../models/whoKnowsBetterQuizState.type';
import {catchError} from 'rxjs';
import {Button} from 'primeng/button';
import {CardModule} from 'primeng/card';


@Component({
  selector: 'app-who-knows-better-quiz',
  standalone : true,
  imports: [CommonModule, Button, CardModule],
  templateUrl: './who-knows-better-quiz.component.html',
  styleUrls: ['./who-knows-better-quiz.component.css'],
})
export class WhoKnowsBetterQuizComponent implements OnInit {

  currentQuestion: Question = {
    id: '',
    question: '',
    groomScore: 0,
    brideScore: 0,
  };
  weddingId: string = '';
  guestId: string = '';

  whoKnowsBetterQuizCompleted: WhoKnowsBetterQuizStateType = {whoKnowsQuizFinished:false};

  selectedAnswer: 'groom' | 'bride' | null = null;
  questionAnswered: boolean = false;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private whoKnowsBetterService: WhoKnowsBetterQuizService
  ) {}


  ngOnInit(): void {
    this.weddingId = this.route.snapshot.paramMap.get('weddingId')!;
    this.guestId = this.route.snapshot.paramMap.get('guestId')!;
    if(this.weddingId && this.guestId) {
      this.loadQuestion();
    }else{
      console.error('weddingId error or guestId error');
    }
  }


  loadQuestion(): void {
    this.whoKnowsBetterService.getWhoKnowsBetterQuizStateByWeddingId(this.weddingId, this.guestId).pipe(
      catchError((error) => {
        console.error('Error:', error);
        throw error;
      })
    ).subscribe((whoKnowsBetterQuizState)=>{
      console.log('whoKnowsBetterQuizState:', whoKnowsBetterQuizState);
      this.whoKnowsBetterQuizCompleted = whoKnowsBetterQuizState;
      if(!this.whoKnowsBetterQuizCompleted.whoKnowsQuizFinished){
        this.whoKnowsBetterService.getQuestion(this.weddingId,this.guestId).pipe(
          catchError((error) => {
            console.error('Error:', error);
            throw error;
          })
        ).subscribe((nextQuestion)=>{
          this.currentQuestion = nextQuestion;
          this.questionAnswered = false;
          this.selectedAnswer = null;
        });
      }else{
        this.router.navigateByUrl('/wedding/${this.weddingId}/${this.guestId}/games/who-knows-better/results');

      }
    });
  }


  submitAnswer(answer:'groom'|'bride'): void {
    console.log('submitAnswer');
    if(!this.questionAnswered){
      this.selectedAnswer = answer;
      console.log("Wybrana odpowiedź:",answer);
      this.questionAnswered = true;

      this.whoKnowsBetterService
        .submitAnswer(this.weddingId, this.guestId, answer)
        .subscribe(() => {
          console.log('Wynik:', answer);
        });
    }
  }

  onClickRoute(): void{
    console.log('onClickRoute');
  }
}
