import {inject, Injectable} from '@angular/core';
import {HttpClient, HttpParams} from '@angular/common/http';
import {Observable} from 'rxjs';
import {ImageType} from '../models/image.type';

@Injectable({
  providedIn: 'root'
})
export class GalleryService {

  constructor(private http:HttpClient) { }

  getGalleryByWeddingId(weddingId: string, guestId: string, sort: string): Observable<[ImageType]> {
    let params = new HttpParams().set('sort', sort);
    return this.http.get<[ImageType]>(`/api/wedding/${weddingId}/${guestId}/gallery`, {params: params});
  }

  postImage(file: File, weddingId: string, guestId: string): Observable<any> {
    const formData = new FormData();

    if (file.size === 0) {
      console.error('File is empty');
    }
    formData.append('image', file, file.name);

    return this.http.post(`/api/wedding/${weddingId}/gallery`, formData);
  }

  changeImageLikes(weddingId : string, guestId: string, weddingImageId : string, liked: boolean): Observable<any> {
    let params = new HttpParams().set('liked', liked.toString());
    return this.http.patch(`/api/wedding/${weddingId}/${guestId}/gallery/${weddingImageId}`, {}, {params: params});
  }
}
